
#ifndef IDE_IPX_SERVICE_H
#define IDE_IPX_SERVICE_H

#include "types.h"
#include <winsock.h>
#include <wsipx.h>
#include "ipx_defn.h"
#define IPX_POLYNOMIAL 0x04c11db7L

#define IPX_WM_RXBUFFER		WM_USER+1  
#define IPX_MAX_BUFFER		4096
#define DEFAULT_TXSOCKET	0x5200
#define DEFAULT_RXSOCKET	0x5201
#define DEFAULT_NETWORK		0x00000A
#define _IPXDLL
#ifdef _IPXDLL
#define IPXAPI __declspec(dllexport) 
#else
#define IPXAPI __declspec(dllimport) 
#endif

class IPXAPI CIpx
{
public:
	virtual  void OnReceive (WPARAM wParam,LPARAM lParam);
    // Default constructor
    CIpx();

    // Destructor
   ~CIpx();

	// Configuration routines 
	void SetNetwork(DWORD network);
	void SetRxSocket(WORD socket);
	void SetTxSocket(WORD socket);

	// Initialize routines 
	ERRCODE CreateRxSocket(void);
	ERRCODE CreateTxSocket(void);

    // Check if initialized 
    BOOL    IsRxInitialized(void); 
	BOOL    IsTxInitialized(void); 

  	// Window messages routines
	void SetHWND (HWND hWnd); 
	void Set_WM_IPX_RXBUFFER (UINT wm); 

	// Set new size of buffers
	ERRCODE SetRxBufferSize (DWORD bufferSize);
	ERRCODE SetTxBufferSize (DWORD bufferSize);

    // close all the sockets
    ERRCODE Deinstall();
	
    // Receive buffer service routines
	UINT BytesInReceiveBuffer () ;
	BOOL  IsReceiveBufferEmpty ();

	// Crc routines 
	UBYTE                Bcc( const UBYTE *buffer, DWORD numToCalc);
	unsigned short       CrcCCIT( unsigned short crc, unsigned short data); 
    unsigned short int   Crc16   (const UBYTE* buffer, DWORD numToCalc);
    DWORD                Crc32   (const UBYTE* buffer, DWORD numToCalc);
	
	// Exchange routines
    ERRCODE PutPacket  (UBYTE* buffer, DWORD numToWrite);
	ERRCODE GetPacket  (UBYTE* buffer, DWORD numToRead); 

private:  
	void	IpxSetup		(void); 	
	void    Gen_crc_32_table(void);
	void	AssignRxSocket(WORD socket);
	void	AssignTxSocket(WORD socket);
	ERRCODE Enable_Broadcasts(SOCKET s,BOOL EnableBrCst);
	ERRCODE CreateDGRMSocket(SOCKET& sock, BYTE* IdSock);
	ERRCODE SendDGRM(SOCKET sock,UBYTE *toSock,
						UBYTE *toNodeAdrr,UBYTE *toNetwork,
									UBYTE *Message,int Len2);
private:
	WORD	m_wVersionRequested;
	WSADATA m_wsaData;
	SOCKET	RxSocket,TxSocket;
	struct sockaddr RecAddr;
	int	   sRecAddr;
	UBYTE	IPX_SEND[2]; 
	UBYTE	IPX_RECV[2]; 
	UBYTE	BROADCAST_NODE[6];

	UBYTE*  m_NewReceiveBuffer;   
	UBYTE*  m_NewTransmitBuffer;   
	DWORD   m_RxBufferSize;
	DWORD   m_TxBufferSize;
	BOOL    m_RxBufferIsNotEmpty;
	HWND	m_hWnd;
	UINT	m_IPX_WM_RXBUFFER;
	BOOL	m_RxInit;
	BOOL	m_TxInit;
	DWORD	m_Network;
    WORD	m_RxSocket;
	WORD	m_TxSocket;
    DWORD	crc_32_table[256];
	UINT	m_Length;	 

};
#endif // IDE_COMM_SERVICE_H